@@ -2,11 +2,11 @@ package Data::Transpose::EmailValid;
 
 use strict;
 use warnings;
-
-use base 'Data::Transpose::Validator::Base';
-
 use Email::Valid;
-
+use Moo;
+extends 'Data::Transpose::Validator::Base';
+use MooX::Types::MooseLike::Base qw(:all);
+use namespace::clean;
 
 =head1 NAME
 
@@ -29,6 +29,25 @@ Data::Transpose::EmailValid - Perl extension to check if a mail is valid (with s
 This module check if the mail is valid, using the L<Email::Valid>
 module. It also provides some additional methods.
 
+=head2 AUTO CORRECTION
+
+This validator corrects common mistakes automatically:
+
+=over 4
+
+=item
+
+C<.ocm> instead of C<.com> as top level domain for C<aol.com>,
+C<gmail.com>, C<hotmail.com> and C<yahoo.com>, e.g. C<tp@gmail.ocm>.
+
+=item
+
+Double dots before top level domain, e.g. C<tp@linuxia..de>.
+
+=back
+
+Please suggest further auto correction examples to us.
+
 =head1 METHODS
 
 =head2 new
@@ -37,61 +56,30 @@ Constructor. It doesn't accept any arguments.
 
 =cut
 
-sub new {
-    my ($class) = @_;
-    my $self = bless {}, $class;
+has _email_valid => (is => 'ro',
+                     isa => Object,
+                     default => sub {
+                         return Email::Valid->new(
+                                                  -fudge   => 1,
+                                                  -mxcheck => 1,
+                                                 );
+                     });
 
-    $self->{email_valid} = Email::Valid->new(
-        -fudge   => 1,
-        -mxcheck => 1,
-    );
+has input => (is => 'rwp',
+              isa => Maybe[Str]);
 
-    return $self;
-}
+has output => (is => 'rwp',
+               isa => Maybe[Str]);
 
-# accessor read only to the Email::Valid object
-
-=head2 email_valid
-
-Accessor to the Email::Valid object
-
-=cut
-
-sub email_valid {
-    my $self = shift;
-    return $self->{email_valid};
-}
 
 =head2 input
 
 Accessor to the input email string.
 
-=cut 
-
-sub input {
-    my ($self, $input) = @_;
-    if (defined $input) {
-        $self->{input} = $input;
-    }
-    return $self->{input};
-}
-
 =head2 output
 
 Accessor to the output email string.
 
-=cut
-
-sub output {
-    my ($self, $output) = @_;
-    if (defined $output) {
-        $self->{output} = $output;
-    }
-    return $self->{output};
-
-
-}
-
 =head2 reset_all 
 
 Clear all the internal data
@@ -102,15 +90,10 @@ Clear all the internal data
 sub reset_all {
     my $self = shift;
     $self->reset_errors;
-    foreach (qw/input output/) {
-        delete $self->{$_} if exists $self->{$_}
-    }
+    $self->_set_input(undef);
+    $self->_set_output(undef);
 }
 
-
-
-
-
 =head2 $obj->is_valid($emailstring);
 
 Returns the email passed if valid, false underwise.
@@ -126,19 +109,19 @@ sub is_valid {
     # overwrite old data
     $self->reset_all;
 
-    $self->input($email);
+    $self->_set_input($email);
 
     # correct common typos # Maybe add an option for this?
     $email = $self->_autocorrect;
 
     # do validation
-    $email = $self->email_valid->address($email);
+    $email = $self->_email_valid->address($email);
     unless ($email) {
-        $self->error($self->email_valid->details);
+        $self->error($self->_email_valid->details);
         return;
     }
 
-    $self->output($email);
+    $self->_set_output($email);
     return $email;
 }
 
@@ -190,6 +173,9 @@ sub _autocorrect {
     foreach (qw/aol gmail hotmail yahoo/) {
         $email =~ s/\b$_\.ocm$/$_.com/;
     }
+    # double dots in domain part
+    $email =~ s/\.\.(\w+)$/.$1/;
+
     # setting the error breaks the retrocompatibility
     # $self->error("typo?");
     return $email;
@@ -201,7 +187,7 @@ Uwe Voelker <uwe@uwevoelker.de>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012-2014 Uwe Voelker <uwe@uwevoelker.de>.
+Copyright 2012-2015 Uwe Voelker <uwe@uwevoelker.de>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
