@@ -742,7 +742,7 @@ undef in that case).
 
 Accessor to set or retrieve the errors (returned as an arrayref of
 hashes). Each element has the key C<field> set to the fieldname and
-the key C<errors> holds the the error list. This, in turn, is a list
+the key C<errors> holds the error list. This, in turn, is a list
 of arrays, where the first element is the error code, and the second
 the human format set by the module (in English). See the method belows
 for a more accessible way for the errors.
@@ -771,7 +771,30 @@ Returns error iterator.
 
 =head2 errors_hash
 
-Returns hash of errors.
+Return an hashref where each key is the name of the error field, and
+the value is an arrayref of hashrefs with two keys, C<name> and
+C<value>.
+
+Example of the returned hash:
+
+          {
+           year => [
+                    {
+                     value => 'Not a number',
+                     name => 'notanumber',
+                    },
+                    {
+                     name => 'notinteger',
+                     value => 'Not an integer',
+                    }
+                   ],
+           mail => [
+                    {
+                     value => 'Missing required field mail',
+                     name => 'required',
+                    }
+                   ],
+          }
 
 =cut
 
